﻿using System;
using System.Data;
using System.Data.OleDb;
using Microsoft.Office;




namespace PhonicsClassLibrary
{
    public interface ICartao
    {
        /// <summary>
        /// Método para inserir um novo cartão pertencente a um
        /// determinado tarifário e um determinado cliente
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="tarifarioID">Código do tarifário</param>
        /// <param name="clienteID">Código do cliente</param>
        /// <param name="numero">Número do cartão</param>
        /// <param name="tipo">Tipo de cartão</param>
        /// <param name="saldo">Saldo do cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e os dados do novo cartão</pre>
        /// <post>Insere um novo cartão na base de dados</post>
        /// <returns>Retorna o estado da operação</returns>
        int Inserir(string username, string pass,
            int tarifarioID, int clienteID,
            int numero, string tipo, float saldo,
            out PhonicsStatusEnum status);

        /// <summary>
        /// Método para cancelar um cartão
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o código do cartão válido</pre>
        /// <post>O estado do cartão ficará como cancelado</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Cancelar(string username, string pass,
            int cartaoID);

        /// <summary>
        /// Método para carregar o saldo de um cartão
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartão</param>
        /// <param name="carregamento">Valor do carregamento</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos, o código do cartão válido e o valor do
        /// carregamento</pre>
        /// <post>Actualiza saldo do cartão</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Carregar(string username, string pass,
            int cartaoID, float carregamento);

        /// <summary>
        /// Método para actualizar os dados de um cartão
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartão</param>
        /// <param name="clienteID">Código do cliente</param>
        /// <param name="tarifarioID">Código do tarifário</param>
        /// <param name="numero">Número do cartão</param>
        /// <param name="tipo">Tipo de cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos, o código do cartão válido e os dados do cartão
        /// </pre>
        /// <post>Actualiza informação do cartão</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Update(string username, string pass,
            int cartaoID, int clienteID, int tarifarioID, int numero,
            string tipo);

        /// <summary>
        /// Método para obter informações de um cartão através do seu
        /// código
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válido e o código do cartão válido</pre>
        /// <post>É retornada toda a informação do cartão com o código
        /// do cartão recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do cartão
        /// </returns>
        DataSet GetInfoByID(string username, string pass, int cartaoID);

        /// <summary>
        /// Método para obter informações de um cartão através do seu
        /// número
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="numero">Número do cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o número do cartão válido</pre>
        /// <post>É retornada toda a informação do cartão com o número
        /// do cartão recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do cartão
        /// </returns>
        DataSet GetInfoByNumero(string username, string pass,
            string numero);       

        /// <summary>
        /// Método para obter informações(cliente, chamadas) de um cartão
        /// através do seu código
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válido e o código do cartão válido</pre>
        /// <post>É retornada toda a informação do cartão com o código
        /// do cartão recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do cartão
        /// </returns>
        DataSet GetByID(string Username, string Pass, int cartaoID);


        /// <summary>
        /// Método para gerar uma factura
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartão</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// e o código do cartão válidos</pre>
        /// <post>É gerada uma factura</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum GerarFactura(string Username, string Pass,
            int cartaoID, out string filename);
        
    }


    internal class Cartao : ICartao
    {
        public Cartao()
        {
        }
        
        //public const string OUTDIR_DOC = @"c:\temp\";

        public int Inserir(string username, string pass,
            int tarifarioID, int clienteID,
            int numero, string tipo, float saldo,
            out PhonicsStatusEnum status)
        {
            int cartaoID = -1;
            OleDbConnection conn = null;

           try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return -1;

                //
                // criar comando SQL a executar
                //
                //INSERIR NA BASE DE DADOS UM NOVO CARTAO
                string sqlCmd = "INSERT INTO Cartao (tarifarioID, clienteID, numero, tipo, saldo, estado) Values (?, ?, ?, ?, ?, ?)";
                OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);

                cmd.Parameters.AddWithValue("tarifarioID", tarifarioID);
                cmd.Parameters.AddWithValue("clienteID", clienteID);
               
                if (numero == 0 || numero == 0)
                    cmd.Parameters.AddWithValue("numero", DBNull.Value);
                else
                    cmd.Parameters.AddWithValue("numero", numero);

                cmd.Parameters.AddWithValue("tipo", tipo);

                if (saldo == 0 || saldo==0)
                    cmd.Parameters.AddWithValue("saldo", DBNull.Value);
                else
                    cmd.Parameters.AddWithValue("saldo", saldo);

                cmd.Parameters.AddWithValue("estado", "Activo");                

                //
                // executar o comando 
                //
                int insRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;
                else
                {
                    //
                    // inseriu registo
                    // obter novo código de id gerado pela BD
                    //
                    OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", conn);
                    cartaoID = (int)idCmd.ExecuteScalar();
                    status = PhonicsStatusEnum.OK;
                }
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return cartaoID;

        }

        public PhonicsStatusEnum Cancelar(string username, string pass,
            int cartaoID)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;

           try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return status;

                //
                // criar comando SQL a executar
                //
                //ALTERAR O ESTADO DO CARTÃO PARA CANCELADO
                string sqlCmd = "UPDATE Cartao SET Estado=? WHERE cartaoID=?";

                OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);

                cmd.Parameters.AddWithValue("Estado", "Cancelado");


                cmd.Parameters.AddWithValue("cartaoID", cartaoID);

                //
                // executar o comando 
                //
                int insRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;

            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            return status;
        }

        public PhonicsStatusEnum Carregar(string username, string pass,
            int cartaoID, float carregamento)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;

          
            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return status;

                //
                // criar comando SQL a executar
                //
                //ALTERAR O VALOR DO SALDO 
                string sqlCmd = "UPDATE Cartao SET Saldo=? WHERE cartaoID=?";

                OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);

                //cmd.Parameters.AddWithValue("Carregamento", carregamento);

                cmd.Parameters.AddWithValue("saldo", carregamento);
                cmd.Parameters.AddWithValue("cartaoID", cartaoID);

                //
                // executar o comando 
                //
                int insRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;

            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //                
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            return status;
        }

        public PhonicsStatusEnum Update(string username, string pass,
            int cartaoID, int clienteID, int tarifarioID, int numero, string tipo)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;

            //
            // validar dados de entrada
            //
           try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return status;

                //
                // criar comando SQL a executar
                //
                //ALTERAR OS DADOS
                string sqlCmd = "UPDATE Cartao SET tarifarioID=?, clienteID=?, numero=?, tipo=? WHERE cartaoID=?";

                OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);

                cmd.Parameters.AddWithValue("tarifarioID", tarifarioID);
                cmd.Parameters.AddWithValue("clienteID", clienteID);
                cmd.Parameters.AddWithValue("numero", numero);
                cmd.Parameters.AddWithValue("tipo", tipo);
                cmd.Parameters.AddWithValue("cartaoID", cartaoID);

                //
                // executar o comando 
                //
                int insRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;

            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            return status;
        }

        public DataSet GetInfoByID(string username, string pass,
            int cartaoID)
        {
           OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexão à base de dados e abrir a conexão
				//
				conn = new OleDbConnection(LigaDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				PhonicsStatusEnum status = LigaDB.Validar(conn, null, username, pass);
				if (status !=PhonicsStatusEnum.OK)
					return null;

				//
				// efectuar pesquisa
				//
				ds = LigaDB.GetById(conn, null, "Cartao", "CartaoID", cartaoID);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO no método <GetByID> da classe 'Cartaor': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexão
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		
        }

        public DataSet GetInfoByNumero(string username, string pass,
            string numero)
        {
            OleDbConnection conn = null;
            DataSet ds = null;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar utilizar
                //
                PhonicsStatusEnum status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return null;

                //
                // efectuar pesquisa
                //
                ds = LigaDB.FindByStringField(conn, "Cartao", "Numero", numero);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <GetInfoByNumero> da classe 'Cartao': " + ex.Message);
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

        public PhonicsStatusEnum GerarFactura(string Username, string Pass, int cartaoID, out string filename)
        {
            //object missingValue = Type.Missing;

            //object uchar = Word.WdUnits.wdCharacter;
            //object uline = Word.WdUnits.wdLine;
            //object ucol = Word.WdUnits.wdColumn;
            //object i1 = 1;
            //object i4 = 4;
            //object bfalse = false;
            //object btrue = true;

            filename = null;

            //// obter factura
            //DataSet dsFactura = GetByID(Username, Pass, cartaoID);
            //if (dsFactura == null)
            //    return PhonicsStatusEnum.ERROR;

            //int nFactura = cartaoID; //devia existir um campo separado para o numero de factura que tem que ser sequencial e sem "buracos";
            //DateTime dataFactura = (DateTime)dsFactura.Tables["Cartao"].Rows[0]["Data"];
            //string customerName = (string)dsFactura.Tables["Cartao"].Rows[0]["Nome"];
            //double totalFactura = (double)dsFactura.Tables["Sales"].Rows[0]["SaleTotal"];

            //// gerar nome de ficheiro
            //object _in_filename = OUTDIR_DOC + "Factura-" + nFactura.ToString() + ".doc";

            //// criar objecto Word
            //Word.Application w = new Word.ApplicationClass();

            //// criar novo documento
            //w.Documents.Add(ref missingValue, ref missingValue, ref missingValue, ref missingValue);

            ////w.Visible = true;	// só para teste/"debug"!!!!!

            //w.Selection.TypeText("Factura nº ");
            //w.Selection.TypeText(nFactura.ToString());
            //w.Selection.TypeParagraph();
            //w.Selection.TypeText("Data : ");
            //w.Selection.TypeText(dataFactura.ToLongDateString());
            //w.Selection.TypeParagraph();
            //w.Selection.TypeText("Cliente : ");
            //w.Selection.TypeText(customerName);
            //w.Selection.TypeParagraph();
            //w.Selection.TypeParagraph();

            ////
            //// criar tabela (cabeçalho)
            ////
            //object tb = Word.WdDefaultTableBehavior.wdWord9TableBehavior;
            //object afb = Word.WdAutoFitBehavior.wdAutoFitWindow;
            //w.ActiveDocument.Tables.Add(w.Selection.Range, 1, 4, ref tb, ref afb);

            //w.Selection.TypeText("Quantidade");
            //w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            //w.Selection.TypeText("Designação");
            //w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            //w.Selection.TypeText("P.U:");
            //w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            //w.Selection.TypeText("sub-total");

            ////
            //// adicionar as linhas de factura
            ////
            //foreach (DataRow dr in dsFactura.Tables["SaleDetails"].Rows)
            //{
            //    // adicionar linha à tabela
            //    w.ActiveDocument.Tables[1].Add(ref missingValue);
            //    w.Selection.MoveDown(ref uline, ref i1, ref missingValue);
            //    w.Selection.MoveLeft(ref uchar, ref i4, ref missingValue);

            //    w.Selection.TypeText(dr["Quantity"].ToString());
            //    w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            //    w.Selection.TypeText(dr["Description"].ToString());
            //    w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            //    w.Selection.TypeText(dr["UnitPrice"].ToString());
            //    w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
            //    w.Selection.TypeText(dr["TotalCost"].ToString());
            //}
            //w.Selection.MoveDown(ref uline, ref i1, ref missingValue);

            //w.Selection.TypeParagraph();
            //w.Selection.TypeText("Total :");
            //w.Selection.TypeText(totalFactura.ToString());
            //w.Selection.TypeParagraph();

            ////
            //// formatar tabela
            ////
            //object tf = Word.WdTableFormat.wdTableFormatProfessional;
            //w.ActiveDocument.Tables[1].AutoFormat(ref tf, ref btrue, ref btrue, ref btrue, ref btrue, ref btrue, ref bfalse, ref bfalse, ref bfalse, ref bfalse);

            //// rodapé da factura
            //w.Selection.TypeParagraph();
            //w.Selection.TypeText("Gerada por ADAV 2006/2007");

            //// gravar ficheiro
            //w.ActiveDocument.SaveAs(ref _in_filename, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue);

            ////terminar Word
            //w.Quit(ref missingValue, ref missingValue, ref missingValue);

            //filename = (string)_in_filename;

            return PhonicsStatusEnum.OK;
        }


        public DataSet GetByID(string Username, string Pass, int cartaoID)
        {
            DataSet ds = null;
            OleDbConnection conn = null;

            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                if (LigaDB.Validar(conn, null, Username, Pass) != PhonicsStatusEnum.OK)
                    return null;

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //
                //FAZER UMA CONSULTA À RELAÇÃO CLIENTE/CARTÃO/CHAMADA
                string sSqlCmd = "SELECT * FROM ConsultaFacturaCliente WHERE clienteID=?";

                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

                oAdapter.SelectCommand.Parameters.AddWithValue("cartaoID", cartaoID);

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, "Cartao");





            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método 'GetById': " + ex);
                ds = null;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }
    }
}