﻿using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;


namespace PhonicsClassLibrary
{
    public interface ICliente
    {
        /// <summary>
        /// Método que insere um novo cliente na base de dados
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="nome">Nome do cliente</param>
        /// <param name="BI">B.I. do cliente</param>
        /// <param name="morada">Morada do cliente</param>
        /// <param name="eMail">Email do cliente</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e os dados do novo cliente</pre>
        /// <post>Insere um novo cliente na base de dados</post>
        /// <returns>Retorna o id no cliente</returns>
        int Inserir(string username, string pass, 
            string nome, int BI, string morada, string eMail, out PhonicsStatusEnum status);

        /// <summary>
        /// Método para actualizar um cliente
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="clienteID">Código do cliente</param>
        /// <param name="morada">Morada do cliente</param>
        /// <param name="eMail">Email do cliente</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o código do cliente válido, morada e email do
        /// cliente</pre>
        /// <post>Actualiza informação do cliente</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Update(string username, string pass,
            int clienteID, string morada, string eMail);

        /// <summary>
        /// Método para obter informações de um cliente através do seu
        /// B.I.
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="BI">B.I. do cliente</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o B.I. válido do cliente</pre>
        /// <post>É retornada toda a informação do cliente com o B.I.
        /// recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do cliente
        /// </returns>
        DataSet GetInfoByBI(string username, string pass, string BI);

        /// <summary>
        /// Método para obter informações de um cliente através do seu
        /// código
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="clienteID">Código do cliente</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o código do cliente válido</pre>
        /// <post>É retornada toda a informação do cliente com o código
        /// de cliente recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do cliente
        /// </returns>
        DataSet GetInfoByID(string username, string pass, int clienteID);


        /// <summary>
        /// Método para obter informações de um cliente através do seu
        /// nome
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="nome">Nome do cliente</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o nome do cliente</pre>
        /// <post>É retornada toda a informação do cliente com o nome
        /// do cliente recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do cliente
        /// </returns>
        DataSet FindByName(string username, string pass, string nome);
    }


    internal class Cliente : ICliente
    {
        public Cliente()
        {
        }
        
        public int Inserir(string username, string pass,
            string nome, int BI, string morada, string eMail,
            out PhonicsStatusEnum status)
        {        
            int clienteID = -1;
			OleDbConnection conn = null;

			//
			// validar dados de entrada
			//
			if (nome == null || nome.Trim().Length == 0)
			{
				status = PhonicsStatusEnum.INVALID_ARGUMENT;
				return -1;
			}

			try
			{	
				//
				// criar objecto de conexão à base de dados e abrir a conexão
				//
				conn = new OleDbConnection(LigaDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = LigaDB.Validar (conn, null, username, pass);
				if (status!= PhonicsStatusEnum.OK)
					return -1;

				//
				// criar comando SQL a executar
				//
                //INSERIR UM NOVO CLIENTE NA BASE DE DADOS
				string sqlCmd = "INSERT INTO Cliente (Nome, BI, Morada,EMail) Values (?, ?, ?, ?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.AddWithValue("nome", nome);
                if (BI == 0 || BI== 0)
                    cmd.Parameters.AddWithValue("BI", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("BI", BI);
				if (morada == null || morada.Length == 0)
                    cmd.Parameters.AddWithValue("morada", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("morada", morada);
				if (eMail == null || eMail.Length == 0)
                    cmd.Parameters.AddWithValue("eMail", DBNull.Value);
				else
                    cmd.Parameters.AddWithValue("eMail", eMail);
		
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// não inseriu registo
					status = PhonicsStatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					// obter novo código de id gerado pela BD
					//
					OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", conn);
					clienteID = (int)idCmd.ExecuteScalar();
					status = PhonicsStatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO no método <AdicionarCliente> da classe 'Cliente': " + ex.Message);
				status = PhonicsStatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexão
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return clienteID;
		}

  
        
        public PhonicsStatusEnum Update(string username, string pass,
            int clienteID, string morada, string eMail)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;

            //
            // validar dados de entrada
            //
            if (morada == null || morada.Trim().Length == 0)
                return PhonicsStatusEnum.INVALID_ARGUMENT;


            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return status;

                //
                // criar comando SQL a executar
                //
                //ALTERAR OS DADOS DO CLIENTE
                string sqlCmd = "UPDATE Cliente SET Morada=?, EMail=? WHERE clienteID=?";

                OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);

                cmd.Parameters.AddWithValue("Morada", morada);

                if (eMail == null || eMail.Length == 0)
                    cmd.Parameters.AddWithValue("eMail", DBNull.Value);
                else
                    cmd.Parameters.AddWithValue("eMail", eMail);

                cmd.Parameters.AddWithValue("clienteID", clienteID);

                //
                // executar o comando 
                //
                int insRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;

            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <Update> da classe 'Cliente': " + ex.Message);
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            return status;
        }
        
        public DataSet GetInfoByBI(string username, string pass,
            string BI) 
        {
            OleDbConnection conn = null;
            DataSet ds = null;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar utilizar
                //
                PhonicsStatusEnum status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return null;

                //
                // efectuar pesquisa
                //
                ds = LigaDB.FindByStringField(conn, "Cliente", "BI", BI);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <GetInfoByBI> da classe 'Cliente': " + ex.Message);
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }
        
        public DataSet GetInfoByID(string username, string pass,
            int clienteID)
        {
            OleDbConnection conn = null;
            DataSet ds = null;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar utilizar
                //
                PhonicsStatusEnum status =LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return null;

                //
                // efectuar pesquisa
                //
                ds = LigaDB.GetById(conn, null, "Cliente", "ClienteID", clienteID);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <GetByID> da classe 'Cliente': " + ex.Message);
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

        public DataSet FindByName(string username, string pass, string nome)
        {
            OleDbConnection conn = null;
            DataSet ds = null;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar utilizar
                //
                PhonicsStatusEnum status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return null;

                //
                // efectuar pesquisa
                //
                ds = LigaDB.FindByStringField(conn, "Cliente", "Nome", nome);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <GetByID> da classe 'Cliente': " + ex.Message);
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }


    }


}